<?php
/**
 * Record Driver Factory Class
 *
 * PHP version 5
 *
 * Copyright (C) Leipzig University Library 2014.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * @category VuFind
 * @package  RecordDrivers
 * @author   André Lahmann <lahmann@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org/wiki/development:plugins:hierarchy_components Wiki
 */
namespace finc\RecordDriver;

use Interop\Container\ContainerInterface;

/**
 * Record Driver Factory Class
 *
 * @category VuFind
 * @package  RecordDrivers
 * @author   André Lahmann <lahmann@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org/wiki/development:plugins:hierarchy_components Wiki
 *
 * @codeCoverageIgnore
 */
class Factory
{
    /**
     * Factory for SolrDefault record driver.
     *
     * @param ContainerInterface $container Service manager.
     *
     * @return SolrDefault
     */
    public static function getSolrDefault(ContainerInterface $container)
    {
        $driver = new SolrDefault(
            $container->get('VuFind\Config')->get('config'),
            null,
            $container->get('VuFind\Config')->get('searches')
        );
        $driver->attachSearchService($container->get('VuFind\Search'));
        return $driver;
    }

    /**
     * Factory for SolrMarc record driver.
     *
     * @param ContainerInterface $container Service manager.
     *
     * @return SolrMarc
     */
    public static function getSolrMarc(ContainerInterface $container)
    {
        $driver = new SolrMarc(
            $container->get('VuFind\Config')->get('config'),
            null,
            $container->get('VuFind\Config')->get('searches')
        );
        $driver->attachILS(
            $container->get('VuFind\ILSConnection'),
            $container->get('VuFind\ILSHoldLogic'),
            $container->get('VuFind\ILSTitleHoldLogic')
        );
        $driver->attachSearchService($container->get('VuFind\Search'));
        return $driver;
    }

    /**
     * Factory for SolrMarcPDA record driver.
     *
     * @param ContainerInterface $container Service manager.
     *
     * @return SolrMarcFincPDA
     */
    public static function getSolrMarcFincPDA(ContainerInterface $container)
    {
        $driver = new SolrMarcFincPDA(
            $container->get('VuFind\Config')->get('config'),
            null,
            $container->get('VuFind\Config')->get('searches')
        );
        //$driver->attachILS(
        //    $container->get('VuFind\ILSConnection'),
        //    $container->get('VuFind\ILSHoldLogic'),
        //    $container->get('VuFind\ILSTitleHoldLogic')
        //);
        $driver->attachSearchService($container->get('VuFind\Search'));
        return $driver;
    }

    /**
     * Factory for SolrMarcRemote record driver.
     *
     * @param ContainerInterface $container Service manager.
     *
     * @return SolrMarcRemote
     */
    public static function getSolrMarcRemote(ContainerInterface $container)
    {
        $driver = new SolrMarcRemote(
            $container->get('VuFind\Config')->get('config'),
            null,
            $container->get('VuFind\Config')->get('searches')
        );
        $driver->attachILS(
            $container->get('VuFind\ILSConnection'),
            $container->get('VuFind\ILSHoldLogic'),
            $container->get('VuFind\ILSTitleHoldLogic')
        );
        $driver->attachSearchService($container->get('VuFind\Search'));
        return $driver;
    }

    /**
     * Factory for SolrAI record driver.
     *
     * @param ContainerInterface $container Service manager.
     *
     * @return SolrAI
     */
    public static function getSolrAI(ContainerInterface $container)
    {
        return new SolrAI(
            $container->get('VuFind\Config')->get('config'),
            $container->get('VuFind\Config')->get('SolrAI'),
            $container->get('VuFind\Config')->get('searches')
        );
    }

    /**
     * Factory for SolrIS record driver.
     *
     * @param ContainerInterface $container Service manager.
     *
     * @return SolrIS
     */
    public static function getSolrIS(ContainerInterface $container)
    {
        // Despite providing recordConfig to AI RecordDriver RecordDriver IS does not
        // need a recordConfig, thus null is provided
        return new SolrIS(
            $container->get('VuFind\Config')->get('config'),
            null,
            null
        );
    }

    /**
     * Factory for SolrMarcRemoteFinc record driver.
     *
     * @param ContainerInterface $container Service manager.
     *
     * @return SolrMarcRemoteFinc
     */
    public static function getSolrMarcRemoteFinc(ContainerInterface $container)
    {
        $driver = new SolrMarcRemoteFinc(
            $container->get('VuFind\Config')->get('config'),
            null,
            $container->get('VuFind\Config')->get('searches')
        );
        $driver->attachILS(
            $container->get('VuFind\ILSConnection'),
            $container->get('VuFind\ILSHoldLogic'),
            $container->get('VuFind\ILSTitleHoldLogic')
        );
        $driver->attachSearchService($container->get('VuFind\Search'));
        return $driver;
    }

    /**
     * Factory for SolrMarcFinc record driver.
     *
     * @param ContainerInterface $container Service manager.
     *
     * @return SolrMarcFinc
     */
    public static function getSolrMarcFinc(ContainerInterface $container)
    {
        $driver = new SolrMarcFinc(
            $container->get('VuFind\Config')->get('config'),
            null,
            $container->get('VuFind\Config')->get('searches')
        );
        $driver->attachILS(
            $container->get('VuFind\ILSConnection'),
            $container->get('VuFind\ILSHoldLogic'),
            $container->get('VuFind\ILSTitleHoldLogic')
        );
        $driver->attachSearchService($container->get('VuFind\Search'));
        return $driver;
    }

    /**
     * Factory for SolrLido record driver.
     *
     * @param ContainerInterface $container Service manager.
     *
     * @return SolrLido
     */
    public static function getSolrLidoNdl(ContainerInterface $container)
    {
        return new SolrLidoNdl(
            $container->get('VuFind\Config')->get('config'),
            null,
            $container->get('VuFind\Config')->get('searches'),
            $container->get('VuFind\DateConverter')
        );
    }

    /**
     * Factory for SolrLidoFinc record driver.
     *
     * @param ContainerInterface $container Service manager.
     *
     * @return SolrLido
     */
    public static function getSolrLido(ContainerInterface $container)
    {
        return new SolrLido(
            $container->get('VuFind\Config')->get('config'),
            null,
            $container->get('VuFind\Config')->get('searches'),
            $container->get('VuFind\DateConverter')
        );
    }

    /**
     * Factory for SolrDico record driver
     *
     * @param ContainerInterface $container Service Manager
     * @return SolrDico
     */
    public static function getSolrDico(ContainerInterface $container)
    {
        return new SolrDico(
            $container->get('VuFind\Config')->get('config'),
            null, //$container->get('VuFind\Config')->get('SolrDico'),
            $container->get('VuFind\Config')->get('searches')
        );
    }
}
