<?php
/**
 * Munger Injection Factory
 *
 * A Delegator Factory that registers several listeners at events triggered by the
 * VuFind\Search service.
 *
 * PHP version 7
 *
 * Copyright (C) Leipzig University Library 2019.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * @category VuFind
 * @package  Finc/Service
 * @author   Dorian Merz <merz@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org Main Page
 */
namespace finc\Service;

use Interop\Container\ContainerInterface;
use VuFindSearch\Query\Query;
use VuFindSearch\Query\QueryGroup;
use VuFindSearch\Service as SearchService;
use Zend\Config\Config;
use Zend\EventManager\EventInterface;
use Zend\ServiceManager\Factory\DelegatorFactoryInterface;

/**
 * Munger Injection Factory
 *
 *
 * @category VuFind
 * @package  Finc/Service
 * @author   Dorian Merz <merz@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org Main Page
 */
class MungerInjectionDelegatorFactory implements DelegatorFactoryInterface
{
    /**
     * @var SearchService
     */
    protected $instance;

    /**
     * @var array names of search handlers for which colons should be escaped
     */
    protected $searches_to_escape;

    /**
     * @var array shard configuration to register in all queries
     */
    protected $shards_to_register;

    /**
     * Creates a delegator of VuFind/Search to register several listeners.
     *
     * @param ContainerInterface $container
     * @param string $name
     * @param callable $callback
     * @param array|null $options
     *
     * @return mixed
     */
    public function __invoke(
        ContainerInterface $container,
        $name,
        callable $callback,
        array $options = null
    ) {
        $instance = call_user_func($callback);

        $searchConfig = $container->get('VuFind\Config')->get('searches');
        $e = $instance->getEventManager()->getSharedManager();

        $handlers = $searchConfig->General->escaped_colon_searches;
        if (!empty($handlers)) {
            $this->searches_to_escape = $handlers->toArray();
            $e->attach(
                'VuFindSearch',
                'pre',
                function (EventInterface $event) {
                    $params = $event->getParams();
                    if (isset($params['query'])) {
                        $params['query'] = $this->escapeColons($params['query']);
                    }
                }
            );
        }
        $shards = $searchConfig->IndexShards->toArray();
        if ($excludedShards = $searchConfig->ShardPreferences->on_user_search_only ?? null) {
            $shards = array_diff_key($shards, array_flip(explode(',', $excludedShards)));
        }
        if (!empty($shards)) {
            $this->shards_to_register = $shards;
            $e->attach('VuFindSearch', 'pre', [$this, 'registerShards']);
        }
        return $instance;
    }

    /**
     * Escapes colons in Queries or recursively in QueryGroups.
     * This prevents queries from being interpreted as advanced queries in Lucene syntax.
     * cf. \VuFindSearch\Backend\Solr\LuceneSyntaxHelper::containsAdvancedLuceneSyntax
     *
     * @param Query|QueryGroup $queryOrGroup
     *
     * @return mixed
     */
    private function escapeColons($queryOrGroup)
    {
        if ($queryOrGroup instanceof QueryGroup) {
            $handler = $queryOrGroup->getReducedHandler();
            if (is_null($handler) || in_array($handler, $this->searches_to_escape)) {
                foreach ($queryOrGroup->getQueries() as $query) {
                    $this->escapeColons($query);
                }
            }
        } elseif (in_array($queryOrGroup->getHandler(), $this->searches_to_escape)) {
            $queryOrGroup->setString(
                // mask whitespaces that follow a colon
                // that avoids the removal of that very colon via
                // \VuFindSearch\Backend\Solr\LuceneSyntaxHelper::normalizeColons
                preg_replace('/(?<=\:)\s/', '\ ', $queryOrGroup->getString())
            );
        }
        return $queryOrGroup;
    }

    /**
     * Event Listener on Search/Pre that registers all configured shards for every
     * search request
     *
     * @param EventInterface $event
     *
     * @return void
     */
    public function registerShards(EventInterface $event)
    {
        $params = $event->getParam('params');
        if (empty($params->get('shards'))) {
            $params->set('shards', implode(',', $this->shards_to_register));
        }
    }
}
